#include once "fltk-c.bi"
#include once "GL/gl.bi"
#include once "GL/glu.bi"

const MENU_HEIGHT     =  30
const LEFT_BOX_WIDTH  =  40
const RIGHT_BOX_WIDTH = 200

type WINDOW_WIDGETS
  as FL_DOUBLE_WINDOWEX ptr mainWin
  as FL_MENU_BAR        ptr menuBar
  as FL_BOX             ptr leftBox
  as FL_GROUP           ptr openglGroup
  as FL_BOX             ptr rightBox
end type

function glViewDrawCB cdecl (self as any ptr) as integer
  const as Fl_Fontsize fontsize = 16
  dim as integer w = Fl_WidgetGetW(self) : if w<1 then return 1
  dim as integer h = Fl_WidgetGetH(self) : if h<1 then return 1

  if Fl_GL_WindowGetContextValid(self)=0 then
    glDrawSetFont FL_HELVETICA,fontsize
    ' reload textures ...
  end if

  if Fl_GL_WindowGetValid(self)=0 then
    glClearColor(0.5,0.5,0.5,1.0)
    glViewport 0,0,w,h
  end  if

  glClear GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT
  glMatrixMode GL_PROJECTION
  glLoadIdentity()
  gluPerspective 60.0,w/h,0.1,5000.0
  glMatrixMode GL_MODELVIEW
  glLoadIdentity
  glEnable(GL_DEPTH_TEST)
  glEnable(GL_LIGHTING)
  ' ...
  ' render 3D stuff
  ' ...

  
  glMatrixMode GL_PROJECTION
  glLoadIdentity()
  glOrtho -1,w,h+1,0,-1,1
  glMatrixMode GL_MODELVIEW
  glLoadIdentity
  glDisable(GL_DEPTH_TEST)
  glDisable(GL_LIGHTING)
  ' ...
  ' render 2D stuff
  ' ...

  glDrawSetColor(Fl_BACKGROUND_COLOR) 
  glBegin(GL_LINES)
    glVertex2f(0,0) : glVertex2f(0,h)
    glVertex2f(0,h) : glVertex2f(w,h)
    glVertex2f(w,h) : glVertex2f(w,0)
    glVertex2f(w,0) : glVertex2f(0,0)
  glEnd()

  glDrawSetColor(FL_WHITE): glDrawStrAti Fl_WidgetGetUserData(self),5,fontsize
  return 1
end function

function glViewResizeCB cdecl (self as any ptr, viewx as integer, viewy as integer, vieww as integer, viewh as integer) as integer
  return 0
end function

function NewglView(x as integer,y as integer,w as integer,h as integer) as Fl_GL_WINDOWEX ptr
  dim as FL_GL_WINDOWEX ptr win = Fl_GL_WindowExNew2(x,y,w,h)
  Fl_GL_WindowSetMode win, FL_MODE_RGB or FL_MODE_DOUBLE or FL_MODE_DEPTH
  Fl_GL_WindowExSetResizeCB win,@glViewResizeCB
  Fl_GL_WindowExSetDrawCB   win,@glViewDrawCB
  return win
end function

function mainWinResizeCB cdecl (self as any ptr, x as integer, y as integer, w as integer, h as integer) as integer
  static as integer oldW=0,oldH=0
  if oldW<>w or oldH<>h then
    oldW=w:oldH=h : h-=MENU_HEIGHT
    dim as WINDOW_WIDGETS ptr ww = Fl_WidgetGetUserData(self)
    Fl_WidgetResize ww->menuBar    ,0             ,0          ,w              ,MENU_HEIGHT
    Fl_WidgetResize ww->leftBox    ,0             ,MENU_HEIGHT,LEFT_BOX_WIDTH ,h
    w-=RIGHT_BOX_WIDTH
    Fl_WidgetResize ww->rightBox   ,w             ,MENU_HEIGHT,RIGHT_BOX_WIDTH,h
    w-=LEFT_BOX_WIDTH
    Fl_WidgetResize ww->openglGroup,LEFT_BOX_WIDTH,MENU_HEIGHT,w              ,h
    return 1
  end if
  return 0
end function

sub mnuQuitCB cdecl (self as Fl_Widget ptr, userdata as any ptr)
  'dim as FL_MENU_ITEM ptr mnuItemQuit = cptr(FL_MENU_ITEM ptr,self)
  flMessageTitle "Edit 3D"
  if flChoice("Do you really want to exit ?","cancel","ok") then
    Fl_WindowHide Fl_WidgetWindow(self)
  end if
end sub

sub mnuAboutCB cdecl (self as Fl_Widget ptr, userdata as any ptr)
  flMessageBox "Edit 3D","Test of some FLTK widgets."
end sub

'
' main
'
dim as WINDOW_WIDGETS ww
ww.mainWin = Fl_Double_WindowExNew(Fl_GetW()*0.75,Fl_GetH()*0.75,"Edit 3D")
Fl_WidgetSetUserData   ww.mainWin,@ww
Fl_Double_WindowExSetResizeCB ww.mainWin,@mainWinResizeCB

var w = Fl_WidgetGetW(ww.mainWin)
var h = Fl_WidgetGetH(ww.mainWin)
ww.menuBar = Fl_Menu_BarNew(0,0,w,MENU_HEIGHT)
'         mnuBar,mnuItem,[shortcut],[@callback],[pUserData],[menuflags])
Fl_Menu_Add(ww.menuBar,"&File",,,,FL_SUBMENU)
  Fl_Menu_Add(ww.menuBar,"File/&Newt",FL_CTRL+asc("n"),,,FL_MENU_INACTIVE)
  Fl_Menu_Add(ww.menuBar,"File/&Load",FL_CTRL+asc("l"),,,FL_MENU_INACTIVE or FL_MENU_DIVIDER)
  Fl_Menu_Add(ww.menuBar,"File/&Save",FL_CTRL+asc("s"),,,FL_MENU_INACTIVE)
  Fl_Menu_Add(ww.menuBar,"File/Save &as ...",         ,,,FL_MENU_INACTIVE or FL_MENU_DIVIDER)
  Fl_Menu_Add(ww.menuBar,"File/&Quit",FL_CTRL+asc("q"),@mnuQuitCB)

Fl_Menu_Add(ww.menuBar,"&Edit",,,,FL_SUBMENU)
  Fl_Menu_Add(ww.menuBar,"Edit/Cut"  ,FL_CTRL+asc("q"),,,FL_MENU_INACTIVE)
  Fl_Menu_Add(ww.menuBar,"Edit/Copy" ,FL_CTRL+asc("c"),,,FL_MENU_INACTIVE or FL_MENU_DIVIDER)
  Fl_Menu_Add(ww.menuBar,"Edit/Paste",FL_CTRL+asc("v"),,,FL_MENU_INACTIVE)

Fl_Menu_Add(ww.menuBar,"&Object",,,,FL_SUBMENU or FL_MENU_INACTIVE)
  ' ...
Fl_Menu_Add(ww.menuBar,"&Views" ,,,,FL_SUBMENU or FL_MENU_INACTIVE)
  ' ...
Fl_Menu_Add(ww.menuBar,"&Render",,,,FL_SUBMENU or FL_MENU_INACTIVE)
  ' ...
Fl_Menu_Add(ww.menuBar,"Help",,,,FL_SUBMENU)
  Fl_Menu_Add(ww.menuBar,"Help/Content ...",FL_F+1,,,FL_MENU_INACTIVE or FL_MENU_DIVIDER)
  Fl_Menu_Add(ww.menuBar,"Help/&About ..." ,FL_F+2,@mnuAboutCB)

h-=MENU_HEIGHT
ww.leftBox  = Fl_BoxNew2(FL_DOWN_BOX,0                ,MENU_HEIGHT,LEFT_BOX_WIDTH ,h)
ww.rightBox = Fl_BoxNew2(FL_DOWN_BOX,w-RIGHT_BOX_WIDTH,MENU_HEIGHT,RIGHT_BOX_WIDTH,h)

w-=LEFT_BOX_WIDTH+RIGHT_BOX_WIDTH
ww.openglGroup = Fl_GroupNew(LEFT_BOX_WIDTH,MENU_HEIGHT,w,h)
Fl_GroupBegin ww.openglGroup
  dim as FL_TILE ptr tile = Fl_TileNew(LEFT_BOX_WIDTH,MENU_HEIGHT,w,h)
  Fl_GroupBegin tile
    w \=2 : h \=2
    Fl_WidgetSetUserData NewGLView(LEFT_BOX_WIDTH  ,MENU_HEIGHT  ,w,h),@"front view"
    Fl_WidgetSetUserData NewGLView(LEFT_BOX_WIDTH+w,MENU_HEIGHT  ,w,h),@"right view"
    Fl_WidgetSetUserData NewGLView(LEFT_BOX_WIDTH  ,MENU_HEIGHT+h,w,h),@"top view"
    Fl_WidgetSetUserData NewGLView(LEFT_BOX_WIDTH+w,MENU_HEIGHT+h,w,h),@"perspective view"
  Fl_GroupEnd tile
Fl_GroupEnd ww.openglGroup
Fl_WindowEnd ww.mainWin
'Fl_GroupSetResizable ww.mainWin,ww.openglGroup
Fl_WindowSizeRange   ww.mainWin,320,180
Fl_WindowShow ww.mainWin
Fl_Run

