#include once "fltk-c.bi"

type MANDEL_T
  declare constructor
  as Fl_Cursor cursor
  as integer Inside
  as integer MaxItera
  as double  Ratio
  as double  iOff
  as double  rOff
  as double  Zoom
  as any ptr pixels
end type
constructor MANDEL_T
  cursor   = FL_CURSOR_DEFAULT
  Inside   = 0
  MaxItera = 1024
  iOff     =  0.0
  rOff     = -0.7
  Zoom     =  1.0
  Ratio    = 2.0/3.1
end constructor


sub Mandel(pixels as any ptr, _
           iMin   as double , _ 
           rMin   as double , _
           iStep  as double , _
           rStep  as double , _
           w      as integer, _
           h      as integer, _
           maxitera as integer)
  static as uinteger ct(255)
  static as uinteger ptr pct=0
  dim as integer x2y2=0
  dim as double i,r
  if pct=0 then
    for c as integer =0 to 255
      ct(c)=rgb(c,c*2,c*3)
    next
    pct=@ct(0)
  else
    dim as uinteger uc=ct(0)
    for c as integer =0 to 254
      ct(c)=ct(c+1)
    next
    ct(255)=uc
  end if

  asm mov edi,[pixels]
  asm mov esi,[pct]
  i=iMin
  For y as integer=1 To h
    r=rMin
    For x as integer=1 To w
      asm
        mov edx,[maxitera]
        xor ecx,ecx
        fld qword ptr [r]
        fld qword ptr [i]
        fldz
        fldz
        fldz
        fldz
        asm_iterastep:
          fld  st(0)
          inc  ecx
          fadd st(0),st(1)
          fmul st(0),st(2)
          fadd st(0),st(5)
          fxch st(1)
          fstp st(0)
          fld  st(3)
          fsub st(0),st(3)
          fadd st(0),st(6)
          fxch st(2)
          fstp st(0)
          fld  st(1)
          fmul st(0)
          fxch st(4)
          fstp st(0)
          fld  st(0)
          fmul st(0)
          fxch st(3)
          fstp st(0)
          cmp  ecx, edx
          je  clear_pixel
          fld  st(2)
          fadd st(0),st(4)
          fistp   dword ptr [x2y2]
          mov eax,dword ptr [x2y2]
          cmp eax,4
          jg exit_itera_loop
        jmp asm_iterastep
        
        clear_pixel:
        mov eax,&HFF000000
        jmp set_pixel

        exit_itera_loop:
        and ecx,255
        mov eax,[esi+ecx*4]
        set_pixel:
        mov [edi],eax
        add edi,4
        fstp st(0)  'x,y2,x2,i,r
        fstp st(0)  'y2,x2,i,r
        fstp st(0)  'x2,i,r
        fstp st(0)  'i,r
        fstp st(0)  'r
        fadd qword ptr [rStep]
        fstp qword ptr [r] 'fpu stack empty
      end asm
    Next
    asm fld  qword ptr [iStep]
    asm fadd qword ptr [i] 
    asm fstp qword ptr [i] 'fpu stack empty
  Next
end sub

function BoxDrawCB cdecl (box as any ptr) as integer
  static as integer ow=0
  static as integer oh=0
  static as any ptr pixels=0
 
  dim as MANDEL_T ptr man=Fl_WidgetGetUserData(box)
  var w=fl_WidgetGetW(box)
  var h=fl_WidgetGetH(box)
  ' resize image data
  if w<>ow or h<>oh then
    man->pixels=reallocate(man->pixels,w*h*4)
    ow=w:oh=h
  end if
  var img = Fl_WidgetGetImage(box)
  if img then Fl_ImageDelete img:img=0
  img = Fl_RGB_ImageNew(man->pixels,w,h,4,0) 
  dim as double rSize = 3.1*man->Zoom
  dim as double iSize = rSize*man->Ratio
  dim as double rStep = rSize/w
  dim as double iStep = iSize/h
  Fl_WindowCursor(Fl_WidgetWindow(box),FL_CURSOR_WAIT)
  mandel man->pixels,man->iOff-iSize*.5,man->rOff-rSize*.5,iStep,rStep,w,h,man->MaxItera
  Fl_WindowCursor(Fl_WidgetWindow(box),man->Cursor)
  Fl_WidgetSetImage box,img
  return 0
end function

function BoxHandleCB cdecl (box as any ptr, event as Fl_Event) as integer
  dim as MANDEL_T ptr man = Fl_WidgetGetUserData(box)
  select case as const event
  case FL_EVENT_ENTER : Fl_WindowCursor(Fl_WidgetWindow(box),FL_CURSOR_CROSS)
                  man->Cursor=FL_CURSOR_CROSS
                  man->inside=1
  case FL_EVENT_LEAVE : Fl_WindowCursor(Fl_WidgetWindow(box),FL_CURSOR_DEFAULT)
                  man->Cursor=FL_CURSOR_DEFAULT
                  man->inside=0
  case else     : return 0
  end select
  return 1
end function

function WindowHandleCB cdecl (win as any ptr, event as Fl_Event) as integer
  dim as Fl_BoxEx ptr box = Fl_WidgetGetUserData(win)
  dim as MANDEL_T ptr man = Fl_WidgetGetUserData(box)
  if man->Inside=0 then return 0
  var x=Fl_WidgetGetX(box)
  var y=Fl_WidgetGetX(box)
  var w=Fl_WidgetGetW(box)
  var h=Fl_WidgetGetH(box)
  select case as const event
  case FL_EVENT_RELEASE
    dim as double sx=(Fl_EventX()-10)-w/2
    dim as double sy=(Fl_EventY()-10)-h/2
    dim as double rSize=3.1*man->Zoom
    dim as double iSize=rSize*man->Ratio
    dim as double rMin=man->rOff-rSize/2
    dim as double iMin=man->iOff-iSize/2
    man->rOff+=rSize/w*sx
    man->iOff+=iSize/h*sy
    Fl_WidgetRedraw box
    return 1
  case FL_EVENT_MOUSEWHEEL
    if Fl_EventDY()>0 then
      man->Zoom*=1.25
      if man->Zoom>2.8 then man->Zoom=2.8
    else
      man->Zoom*=0.75
      if man->Zoom<3.5e-14 then 
        
        man->Zoom=3.5e-14
      end if
    end if
    'Fl_WindowCopyLabel win,"zoom " & man->Zoom
    Fl_WidgetRedraw box
    return 1
  end select
  return 0
end function

'
' main
'
' get half screen size
var w = Fl_GetW()\2
var h = Fl_GetH()\2
' create centered extended window
var win = Fl_Double_WindowExNew2(w-w\2,h-h\2,w,h,"Mandelbrot.bas")
Fl_Double_WindowExSetHandleCB win,@WindowHandleCB
' use the client region as box widget
var box = Fl_BoxExNew(10,10,w-20,h-20)
Fl_BoxExSetDrawCB   box,@BoxDrawCB
Fl_BoxExSetHandleCB box,@BoxHandleCB
var man = new MANDEL_T 
Fl_WidgetSetUserData box,man
Fl_WidgetSetUserData win,box
' set window min size
Fl_WindowSizeRange win,320,240
' make window and box Resizable
Fl_GroupSetResizable win,box
' put it on desktop
Fl_WindowShow win
' enter the main loop
Fl_Run

' free the pixel buffer
if man->pixels then deallocate man->pixels

