#include once "fltk-c.bi"

#define __DEBUG__
#ifdef __DEBUG__
#define dprint(_msg_) : open err for output as #99 : print #99,_msg_ : close #99 :
#else
#define dprint(_msg_) :
#endif



' ###############
' # FLTK_WIDGET #
' ###############
type FLTK_WIDGET extends object
  declare sub SetClass(pClass as any ptr)
  declare sub color (c as uinteger)
  declare function color as uinteger
  private:
  as FL_WIDGET ptr pWidget
end type
sub FLTK_WIDGET.SetClass(pClass as any ptr)
  dprint("FLTK_WIDGET.SetClass")
  pWidget=pClass
end sub
sub FLTK_WIDGET.color (c as uinteger)
  dprint("FLTK_WIDGET.color")
  Fl_WidgetSetColor(pWidget,c)
end sub

function FLTK_WIDGET.color as uinteger
  dprint("FLTK_WIDGET.color()")
  return Fl_WidgetGetColor(pWidget)
end function

' ###############
' # FLTK_BUTTON #
' ###############
type FLTK_BUTTON extends FLTK_WIDGET
  declare constructor
  declare constructor(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0)
  declare sub SetClass(pClass as any ptr)
  private:
  as Fl_Button ptr pButton
end type
constructor FLTK_BUTTON
end constructor
constructor FLTK_BUTTON(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr)
  dprint("FLTK_BUTTON()")
  SetClass Fl_ButtonNew(x,y,w,h,l)
end constructor
sub FLTK_BUTTON.SetClass(pClass as any ptr)
  pButton = pClass : base.SetClass(pButton)
end sub


' ##############
' # FLTK_GROUP #
' ##############
type FLTK_GROUP extends FLTK_WIDGET
  declare constructor
  declare constructor (x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0)
  declare sub SetClass(pClass as any ptr)
  private:
  as FL_GROUP ptr pGroup
end type
constructor FLTK_GROUP
end constructor
constructor FLTK_GROUP(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr)
  dprint("FLTK_GROUP()")
  SetClass Fl_GroupNew(x,y,w,h,l)
end constructor
sub FLTK_GROUP.SetClass(pClass as any ptr)
  dprint("FLTK_GROUP.SetClass")
  pGroup=pClass : base.SetClass(pGroup)
end sub

' ###############
' # FLTK_WINDOW #
' ###############
type FLTK_WINDOW extends FLTK_GROUP
  public:
  declare destructor
  declare constructor (w as integer=-1,h as integer=-1,l as const zstring ptr=0)
  declare constructor (x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0)
  ' behavior
  declare sub show
  ' properties
  declare sub color (c as uinteger)
  declare function color as uinteger
  ' childs
  declare function Group(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  declare function Button(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  private:
  const FLTK_GROUP_BIT  = 1 shl 29
  const FLTK_WIDGET_BIT = 1 shl 28
  
  declare sub create(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr)
  declare function AddChild(pChild as any ptr) as integer
  as FL_WINDOW ptr pWindow
  as integer     nChilds,cChild
  as FLTK_WIDGET ptr ptr Childs
end type

sub FLTK_WINDOW.create(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr)
  if w=-1 then w=Fl_GetW()\2
  if h=-1 then h=Fl_GetH()\2
  if x=-1 and y=-1 then
    pWindow = Fl_WindowNew(w,h,l)
  else
    pWindow = Fl_WindowNew2(x,y,w,h,l)
  end if
end sub

destructor FLTK_WINDOW
  dprint("FLTK_WINDOW~")
  if Childs then deallocate Childs
  sleep 1000,0
end destructor

constructor FLTK_WINDOW(w as integer,h as integer,l as const zstring ptr)
  dprint("FLTK_WINDOW()")
  create -1,-1,w,h,l : base.SetClass pWindow
end constructor

constructor FLTK_WINDOW(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr)
  dprint("FLTK_WINDOW()")
  create x,y,w,h,l : base.SetClass pWindow 
end constructor

function FLTK_WINDOW.AddChild(pChild as any ptr) as integer
  dim as integer index=-1
  if nChilds>0 then
    for i as integer=0 to nChilds-1
      if Childs[i]=0 then index=i:exit for
    next
  end if
  if index=-1 then
    index=nChilds:nChilds+=1
    Childs=reallocate(childs,sizeof(any ptr)*nChilds)
  end if
  Childs[index]=pChild
  cChild=index
  return index
end function

sub FLTK_WINDOW.show
  Fl_WindowShow pWindow
end sub

sub FLTK_WINDOW.color (c as uinteger)
  dprint("FLTK_WINDOW.color()")
  base.color(c)
end sub

function FLTK_WINDOW.color as uinteger
  dprint("FLTK_WINDOW.color()")
  return base.color()
end function

function FLTK_WINDOW.Group(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr) as integer
  dprint("FLTK_WINDOW.GROUP()")
  return AddChild(new FLTK_GROUP(x,y,w,h,l)) or FLTK_GROUP_BIT
end function

function FLTK_WINDOW.Button(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr) as integer
  dprint("FLTK_WINDOW.BUTTON()")
  return AddChild(new FLTK_BUTTON(x,y,w,h,l)) or FLTK_WIDGET_BIT
end function


' ########
' # FLTK #
' ########
type FLTK
  declare constructor
  declare destructor
  declare function window (w as integer=-1,h as integer=-1,l as const zstring ptr=0) as integer
  declare function window (x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  declare function group  (x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  declare function button (x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  declare function run as integer
  private:
  const FLTK_WINDOW_BIT = (1 shl 30)
  declare function AddChild(pChild as FLTK_WINDOW ptr) as integer
  as integer nChilds,cChild
  as FLTK_WINDOW ptr ptr Childs
end type
dim shared as FLTK gui

constructor FLTK
  cChild=-1  ' no [c]urent child
end constructor

destructor FLTK
  dprint("FLTK~")
  if Childs then
    if nChilds>0 then
      for index as integer=0 to nChilds-1
        if Childs[index] then delete Childs[index]
      next
    end if
    deallocate Childs
  end if
  sleep 1000,1
end destructor

function FLTK.AddChild(pChild as FLTK_WINDOW ptr) as integer
  dim as integer index=-1
  if nChilds>0 then
    for i as integer=0 to nChilds-1
      if Childs[i]=0 then index=i:exit for
    next
  end if
  if index=-1 then
    index=nChilds:nChilds+=1
    Childs=reallocate(childs,sizeof(any ptr)*nChilds)
  end if
  Childs[index]=pChild
  index or=FLTK_WINDOW_BIT
  cChild=index
  return index
end function

function FLTK.window (w as integer=-1,h as integer=-1,l as const zstring ptr=0) as integer
  return AddChild(new FLTK_WINDOW(w,h,l))
end function
function FLTK.window (x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  return AddChild(new FLTK_WINDOW(x,y,w,h,l))
end function
function FLTK.group(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  return Childs[cChild]->group(x,y,w,h,l)
end function
function FLTK.button(x as integer,y as integer,w as integer,h as integer,l as const zstring ptr=0) as integer
  return Childs[cChild]->button(x,y,w,h,l)
end function

function FLTK.run as integer
  if nChilds>0 then Childs[cChild]->show
  return Fl_Run()
end function
