'  ###############
' # plug-ds.bas #
'###############
' Copyright 2014 by D.J.Peters (Joshy)
' d.j.peters@web.de

#include once "plug.bi"
#include once "plug-ds.bi"

#define max_buffers 64
#define min_frames  512


type DS
  as FBS_PLUG        Plug
  as zstring * 128   LastError
  as any ptr         FillThreadID
  as integer         FillEnd
#if 0
  as any ptr         FillCondition,FillMutex
  as any ptr         WriteThreadID,WriteCondition,WriteMutex
  as integer         FillEnd,WriteEnd
#endif
  as DS_DEVICE_LIST      DeviceList
  as LPDIRECTSOUND       pDirectSound

  as LPDIRECTSOUNDBUFFER pPrimaryBuffer
  as DSBUFFERDESC        PrimaryBufferDesc
  as DSBCAPS             PrimaryCaps
  as WAVEFORMATEX        PrimaryWaveFormat

  as LPDIRECTSOUNDBUFFER pSecondaryBuffer
  as DSBCAPS             SecondaryCaps
  as DSBUFFERDESC        SecondaryBufferDesc
  as WAVEFORMATEX        SecondaryWaveFormat

  as LPDIRECTSOUNDNOTIFY pSoundNotify
  as LPDSBPOSITIONNOTIFY pPositionNotify
  as LPHANDLE            phEvents

  as integer             nRate
  as integer             nBits
  as integer             nChannels
  as integer             nBuffers
  as integer             nFrames
  as integer             nFramesize
  as integer             Playing
  as integer             Locked
end type

dim shared as DS _Me

sub _init() constructor
  dprint("ds:()")
  ' Initialize COM
  CoInitialize(NULL)
  _Me.Plug.Plugname="DirectSound"
end sub

sub _exit() destructor
  dprint("ds:~")
 ' Uninitialize COM
  CoUninitialize()
  #ifdef DEBUG
  sleep 2000
  #endif
end sub


private _
sub FillThread(byval unused as any ptr)
  dim as integer i,lpArg
  dprint("ds:FillThread()")
  dim as ubyte ptr pBuffer=new ubyte[_Me.Plug.Buffersize]
  _Me.Plug.lpCurentBuffer=pBuffer
  lpArg=cint(@_Me.Plug)
  _Me.FillEnd=0 ' signal thread loop
  _Me.Plug.FillBuffer(lpArg) ' get first buffer
  while _Me.FillEnd=0
    dim as DWORD ret = MsgWaitForMultipleObjects(_Me.nBuffers,_Me.pHEvents,0,INFINITE,QS_POSTMESSAGE)
    if (ret<>WAIT_TIMEOUT) andalso (_Me.FillEnd=0) then
      dim as integer BufferIndex=(ret+1) mod _Me.Plug.nBuffers
      dim as integer position=BufferIndex*_Me.Plug.Buffersize
      dim as ubyte ptr pAudio1,pAudio2
      dim as integer nSize1,nSize2,IsLocked
      dim as HRESULT LockRet=IDirectSoundBuffer_Lock(_Me.pSecondaryBuffer,position,_Me.Plug.Buffersize,@pAudio1,@nSize1,@pAudio2,@nSize2,0)
      if LockRet=DSERR_BUFFERLOST then
        dprint("ds: buffer lost !")
        if IDirectSoundBuffer_Restore(_Me.pSecondaryBuffer)=DS_OK then
          if IDirectSoundBuffer_Lock(_Me.pSecondaryBuffer,position,_Me.Plug.Buffersize,@pAudio1,@nSize1,@pAudio2,@nSize2,0)=DS_OK then
            IsLocked=-1
          end if
        end if
      elseif LockRet=DS_OK then
        IsLocked=-1
      end if
      if IsLocked then
        if pAudio1 then memcpy(pAudio1,_Me.Plug.lpCurentBuffer,nSize1)
        if pAudio2 then memcpy(pAudio2,_Me.Plug.lpCurentBuffer+nSize1,nSize2)
        IDirectSoundBuffer_Unlock(_Me.pSecondaryBuffer,pAudio1,nSize1,pAudio2,nSize2)
        IsLocked=0
        _Me.Plug.FillBuffer(lpArg)
      else
        dprint("ds: could not lock the buffer !")
      end if
    end if
  wend
  if pBuffer then
    delete [] pBuffer
  end if
  dprint("ds:FillThread~")
end sub


function plug_error() as string export
  dim tmp as string
  tmp=_Me.LastError
  return tmp
end function

function plug_isany(byref Plug as FBS_PLUG) as fbsboolean export
  dprint("ds:plug_isany")
  if _Me.DeviceList.nDevices>0 then 
    dprint("ds:warning: isany() called more than once !")
    return true
  end if
  Plug.Plugname=_Me.Plug.Plugname
  _Me.Plug.DeviceName=""

  ' get list of none emulated DirectSound devices
  DirectSoundEnumerate(@DSDeviceEnumCB,@_Me.DeviceList)

  if _Me.DeviceList.nDevices<1 then 
    _Me.LastError="ds:error no devices!"
    dprint(_Me.LastError)
    return false
  end if 
  return true
end function

function plug_start() as fbsboolean export
  dprint("dd:plug_start")
  if _Me.pDirectSound=NULL then
    _Me.LastError="ds:error no device initialized !"
    dprint(_Me.LastError)
    return false
  end if
  ' plug is running
  if (_Me.FillThreadID<>NULL) then 
    _Me.LastError="ds:warniung: thread's are running."
    dprint(_Me.LastError)
    return false
  end if

  _Me.FillEnd       =1
  _Me.FillThreadID  =ThreadCreate(@FillThread)
  while _Me.FillEnd=1:sleep(10,1):wend
  
  if (IDirectSoundBuffer_Play(_Me.pSecondaryBuffer,0,0,DSBPLAY_LOOPING)=DS_OK) then
    dim as DWORD status
    if (IDirectSoundBuffer_GetStatus(_Me.pSecondaryBuffer,@status)=DS_OK) then
      if ((status and DSBSTATUS_PLAYING)<>0) then return true
    end if
  end if
  _Me.LastError="ds:error can't looping buffer!"
  dprint(_Me.LastError)
  return false
end function

function plug_stop() as fbsboolean export
  dprint("ds:plug_stop()")
  if _Me.pDirectSound=NULL then
    _Me.LastError="ds:warning no open device."
    dprint(_Me.LastError)
  end if

  if (_Me.FillThreadID=NULL) then 
    _Me.LastError="ds:warning no running threads."
    dprint(_Me.LastError)
    return true
  end if

  ' signal end of thread
  _Me.FillEnd=1
  Threadwait _Me.FillThreadID
  _Me.FillThreadID=NULL

  ' stop secondary buffers
  if _Me.pSecondaryBuffer<>NULL then
    dim as DWORD status
    if (IDirectSoundBuffer_GetStatus(_Me.pSecondaryBuffer,@status)=DS_OK) then
      if ((status and DSBSTATUS_PLAYING)<>0) then
        dprint("ds: stop secondary buffer")
        IDirectSoundBuffer_Stop(_Me.pSecondaryBuffer)
      end if
    end if
  end if


  dprint("ds:plug_stop~")
  return true
end function

function plug_exit() as fbsboolean export
  dim as integer i,ret
  dprint("ds:plug_exit()")

  if _Me.pDirectSound=NULL then
    _Me.LastError="ds:warning no open device."
    dprint(_Me.LastError)
    return true
  end if

  if (_Me.FillThreadID<>NULL) then
    ret=plug_stop()
  end if

  ' delete event handles
  if (_Me.phEvents<>NULL) then
    for i as integer=0 to _Me.nBuffers-1
      if _Me.phEvents[i]<>NULL then 
        'dprint("CloseHandle(phEvents[" & i & "])")
        CloseHandle(_Me.phEvents[i])
        _Me.phEvents[i]=NULL
      end if
    next
    delete[] _Me.phEvents
  end if

  ' free pPositionNotify
  if (_Me.pPositionNotify<>NULL) then
    delete [] _Me.pPositionNotify
    _Me.pPositionNotify=0
  end if

  ' release sound notify
  if (_Me.pSoundNotify<>NULL) then
    '  dprint("IDirectSoundNotify_Release(pSoundNotify)")
    IDirectSoundNotify_Release(_Me.pSoundNotify)
    _Me.pSoundNotify=0
  end if

  ' release secondary buffer
  if (_Me.pSecondaryBuffer<>NULL) then
    ' dprint("IDirectSoundBuffer_Release(pSecondaryBuffer)")
    IDirectSoundBuffer_Release(_Me.pSecondaryBuffer)
    _Me.pSecondaryBuffer=0
  end if

  ' release direct sound
  ' dprint("IDirectSound_Release(pDirectSound)")
  IDirectSound_Release(_Me.pDirectSound)
  _Me.pDirectSound=0


  dprint("ds:plug_exit~")
  return true
end function


function plug_init(byref Plug as FBS_PLUG) as fbsboolean export
  dprint("ds:plug_init")

  if (_Me.pDirectSound<>NULL) then
    _Me.LastError="ds:error: device is open!"
    dprint(_Me.LastError)
    return false
  end if

  if (_Me.DeviceList.nDevices<1) then
    _Me.LastError="ds:error no devices!"
    dprint(_Me.LastError)
    return false
  end if

  if Plug.DeviceIndex<0 then
    Plug.DeviceIndex=0
  elseif Plug.DeviceIndex>=_Me.DeviceList.nDevices then
    dprint("ds:warning device index out of range.")
    Plug.DeviceIndex=Plug.DeviceIndex mod _Me.DeviceList.nDevices
  end if

  _Me.nRate     = Plug.fmt.nRate
  _Me.nBits     = Plug.fmt.nBits
  _Me.nChannels = Plug.fmt.nChannels
  _Me.nBuffers  = Plug.nBuffers
  _Me.nFrames   = Plug.nFrames


  if     _Me.nRate< 11026 then
    _Me.nRate=11025
  elseif _Me.nRate< 22051 then
    _Me.nRate=22050
  elseif _Me.nRate< 32001 then
    _Me.nRate=32000
  elseif _Me.nRate< 38001 then
    _Me.nRate=38000
  elseif _Me.nRate< 44101 then
    _Me.nRate=44100
  elseif _Me.nRate< 48001 then
    _Me.nRate=48000
  elseif _Me.nRate<100000 then
    _Me.nRate=96000
  else
    _Me.nRate=100000
  end if

  if     _Me.nBuffers< 2 then
    _Me.nBuffers=2
  elseif _Me.nBuffers>64 then
    _Me.nBuffers=64
  end if

  if     _Me.nFrames<  513 then
    _Me.nFrames=512
  elseif _Me.nFrames< 1025 then
    _Me.nFrames=1024
  elseif _Me.nFrames< 2049 then
    _Me.nFrames=2048
  elseif _Me.nFrames< 4097 then
    _Me.nFrames=4096
  elseif _Me.nFrames< 8193 then
    _Me.nFrames=8192
  elseif _Me.nFrames<16385 then
    _Me.nFrames=16384
  else
    _Me.nFrames=32768
  end if

  dim as fbsboolean found = 0
  ' create device
  if (DirectSoundCreate(_Me.DeviceList.pDevices[Plug.DeviceIndex]->getGuid(),@_Me.pDirectSound,NULL)=DS_OK) then
    ' set cooperative level
    if (IDirectSound_SetCooperativeLevel(_Me.pDirectSound,GetForegroundWindow(),DSSCL_PRIORITY)=DS_OK) then
      ' create primary sound buffer
      with _Me.PrimaryBufferDesc
        .dwSize       =sizeof(DSBUFFERDESC)
        .dwFlags      =DSBCAPS_PRIMARYBUFFER
        .dwBufferBytes=0
        .lpwfxFormat  =NULL
      end with
      if (IDirectSound_CreateSoundBuffer(_Me.pDirectSound,@_Me.PrimaryBufferDesc,@_Me.pPrimaryBuffer,NULL)=DS_OK) then
        dprint("create primary buffer ok")
        ' get buffer caps
        _Me.PrimaryCaps.dwSize = sizeof(DSBCAPS) 
        if (IDirectSoundBuffer_GetCaps(_Me.pPrimaryBuffer,@_Me.PrimaryCaps)=DS_OK) then
          dim as integer hwbuffersize =  _Me.PrimaryCaps.dwBufferBytes
          dprint("get primary caps hw buffersize " & hwbuffersize & " ok")
          setWaveFormatEx(_Me.PrimaryWaveFormat,_Me.nRate,_Me.nBits,_Me.nChannels)
          ' create secondary sound buffer
          setWaveFormatEx(_Me.SecondaryWaveFormat,_Me.nRate,_Me.nBits,_Me.nChannels)
          with _Me.SecondaryBufferDesc
            .dwSize       =sizeof(DSBUFFERDESC)
            .dwFlags      =DSBCAPS_CTRLPOSITIONNOTIFY or DSBCAPS_GETCURRENTPOSITION2
            .dwBufferBytes=_Me.nBuffers*_Me.nFrames*4
            while .dwBufferBytes<hwbuffersize
             _Me.nFrames shl=1
             .dwBufferBytes=_Me.nBuffers*_Me.nFrames*4
            wend
            .lpwfxFormat  =@_Me.SecondaryWaveFormat
          end with
          if IDirectSoundBuffer_Release(_Me.pPrimaryBuffer)=DS_OK then
            dprint("release primary buffer ok")
            _Me.pPrimaryBuffer=0
          end if
          if (IDirectSound_CreateSoundBuffer(_Me.pDirectSound,@_Me.SecondaryBufferDesc,@_Me.pSecondaryBuffer,NULL)=DS_OK) then
            dprint("create secondary buffer ok")
            ' get buffer caps
            _Me.SecondaryCaps.dwSize = sizeof(DSBCAPS) 
            if (IDirectSoundBuffer_GetCaps(_Me.pSecondaryBuffer,@_Me.SecondaryCaps)=DS_OK) then
              dim as integer swbuffersize=_Me.SecondaryCaps.dwBufferBytes
              dprint("get secondary caps sw buffersize " & swbuffersize & " ok")
              ' get curent format
              if IDirectSoundBuffer_GetFormat(_Me.pSecondaryBuffer,@_Me.SecondaryWaveFormat,sizeof(WAVEFORMATEX),NULL)=DS_OK then
                dprint("get secondary format ok")
                _Me.nRate      = _Me.SecondaryWaveFormat.nSamplesPerSec
                _Me.nBits      = _Me.SecondaryWaveFormat.wBitsPerSample 
                _Me.nChannels  = _Me.SecondaryWaveFormat.nChannels
                _Me.nFramesize = _Me.SecondaryWaveFormat.nBlockAlign
                _Me.nFrames    = swbuffersize\(_Me.nFramesize*_Me.nBuffers)
               
                if IDirectSoundBuffer_QueryInterface(_Me.pSecondaryBuffer,@IID_IDirectSoundNotify,@_Me.pSoundNotify)=S_OK then
                  dprint("get IID_IDirectSoundNotify ok")
                  _Me.phEvents = new HANDLE[_Me.nBuffers]
                  _Me.pPositionNotify = new DSBPOSITIONNOTIFY[_Me.nBuffers]
                  for i as integer=0 to _Me.nBuffers-1
                    _Me.phEvents[i]=CreateEvent(NULL,FALSE,FALSE,NULL)
                    _Me.pPositionNotify[i].dwOffset=i*_Me.nFrames*_Me.nFramesize
                    _Me.pPositionNotify[i].hEventNotify=_Me.phEvents[i]
                    dprint("pPositionNotify[" & i & "].dwOffset = " & _Me.pPositionNotify[i].dwOffset)
                  next
                  if IDirectSoundNotify_SetNotificationPositions(_Me.pSoundNotify,_Me.nBuffers,_Me.pPositionNotify)=DS_OK then
                    dprint("IDirectSoundNotify_SetNotificationPositions ok")
                    found=true
                  end if
                end if ' IID_IDirectSoundNotify
              end if ' SecondaryWaveFormat
            end if ' SecondaryCaps
          end if ' pSecondaryBuffer
        end if ' PrimaryCaps
      end if ' pPrimaryBuffer
    end if ' DSSCL_PRIORITY
  end if ' pDirectSound
  if found=false then
    _Me.LastError="ds:error can't create device[" & Plug.DeviceIndex & !"]\n" & *_Me.DeviceList.pDevices[Plug.DeviceIndex]
    return false
  end if
  
  ' fill callers struct
  Plug.fmt.nRate     = _Me.nRate
  Plug.fmt.nBits     = _Me.nBits
  Plug.fmt.nChannels = _Me.nChannels
  Plug.fmt.signed    = true
  Plug.nBuffers      = _Me.nBuffers
  Plug.nFrames       = _Me.nFrames
  Plug.Framesize     = _Me.nFramesize
  Plug.Buffersize    = _Me.nFrames*_Me.nFramesize
  Plug.Plugname      = _Me.Plug.Plugname
  Plug.DeviceName    = *_Me.DeviceList.pDevices[Plug.DeviceIndex]->getDescription()
  ' fill own struct
  _Me.Plug.fmt.nRate     = _Me.nRate
  _Me.Plug.fmt.nBits     = _Me.nBits
  _Me.Plug.fmt.nChannels = _Me.nChannels
  _Me.Plug.fmt.signed    = true
  _Me.Plug.nBuffers      = _Me.nBuffers
  _Me.Plug.nFrames       = _Me.nFrames
  _Me.Plug.Framesize     = _Me.nFramesize
  _Me.Plug.Buffersize    = _Me.nFrames*_Me.nFramesize
  _Me.Plug.FillBuffer    = Plug.FillBuffer

  return true
end function

