#ifndef __IM_BI__
#define __IM_BI__

' see copyright notice at the file iup.txt

#include once "iup.bi"

#inclib "im"

enum imDataType
  IM_BYTE,   /'*< "unsigned char". 1 byte from 0 to 255.                  '/
  IM_USHORT, /'*< "unsigned short". 2 bytes from 0 to 65,535.             '/
  IM_INT,    /'*< "int". 4 bytes from -2,147,483,648 to 2,147,483,647.    '/
  IM_FLOAT,  /'*< "float". 4 bytes single precision IEEE floating point.  '/
  IM_CFLOAT  /'*< complex "float". 2 float values in sequence, real and imaginary parts.   '/
end enum

/'* Image color mode color space descriptors (first byte). \n
 * See also \ref colormodeutl.
 * \ingroup imagerep '/
enum imColorSpace
  IM_RGB,    /'*< Red, Green and Blue (nonlinear).              '/
  IM_MAP,    /'*< Indexed by RGB color map (data_type=IM_BYTE). '/
  IM_GRAY,   /'*< Shades of gray, luma (nonlinear Luminance), or an intensity value that is not related to color. '/
  IM_BINARY, /'*< Indexed by 2 colors: black (0) and white (1) (data_type=IM_BYTE).     '/
  IM_CMYK,   /'*< Cian, Magenta, Yellow and Black (nonlinear).                          '/
  IM_YCBCR,  /'*< ITU-R 601 Y'CbCr. Y' is luma (nonlinear Luminance).                   '/
  IM_LAB,    /'*< CIE L*a*b*. L* is Lightness (nonlinear Luminance, nearly perceptually uniform). '/
  IM_LUV,    /'*< CIE L*u*v*. L* is Lightness (nonlinear Luminance, nearly perceptually uniform). '/
  IM_XYZ     /'*< CIE XYZ. Linear Light Tristimulus, Y is linear Luminance.             '/
end enum

/'* Image color mode configuration/extra descriptors (1 bit each in the second byte). \n
 * See also \ref colormodeutl.
 * \ingroup imagerep '/
enum imColorModeConfig
  IM_ALPHA    = &H100,  /'*< adds an Alpha channel '/
  IM_PACKED   = &H200,  /'*< packed components (rgbrgbrgb...) '/
  IM_TOPDOWN  = &H400   /'*< orientation from top down to bottom '/
end enum

/'* File Access Error Codes ingroup file '/
enum imErrorCodes
  IM_ERR_NONE,     /'*< No error. '/
  IM_ERR_OPEN,     /'*< Error while opening the file (read or write). '/
  IM_ERR_ACCESS,   /'*< Error while accessing the file (read or write). '/
  IM_ERR_FORMAT,   /'*< Invalid or unrecognized file format. '/
  IM_ERR_DATA,     /'*< Invalid or unsupported data. '/
  IM_ERR_COMPRESS, /'*< Invalid or unsupported compression. '/
  IM_ERR_MEM,      /'*< Insuficient memory '/
  IM_ERR_COUNTER   /'*< Interrupted by the counter '/
end enum

/' Internal Image File Structure. '/
type _imFile as any : type imFile as _imFile

extern "c"

_DF_ imFileOpen(file_name as const zstring ptr, error as integer ptr) as imFile ptr
_DF_ imFileOpenAs(file_name as const zstring ptr, format as const zstring ptr, error as integer ptr) as imFile ptr
_DF_ imFileNew(file_name as const zstring ptr, format as const zstring ptr, error as integer ptr) as imFile ptr
_DS_ imFileClose(ifile as imfile ptr)  
_DF_ imFileHandle(ifile as imfile ptr, index as integer) as  any ptr  
_DS_ imFileGetInfo(ifile as imfile ptr, format as zstring ptr, compression as zstring ptr, image_count as integer ptr)
_DS_ imFileSetInfo(ifile as imfile ptr, compression as const zstring ptr)
_DS_ imFileSetAttribute(ifile as imfile ptr, attrib as const zstring ptr, data_type as integer, count as integer, data as const any ptr)
_DF_ imFileGetAttribute(ifile as imfile ptr, attrib as const zstring ptr, data_type as integer ptr, count as integer ptr) as const any ptr
_DS_ imFileGetAttributeList(ifile as imfile ptr, attrib as zstring ptr ptr, attrib_count as integer ptr)
_DS_ imFileGetPalette(ifile as imfile ptr, palette as long ptr, palette_count as integer ptr)
_DS_ imFileSetPalette(ifile as imfile ptr, palette as long ptr, palette_count as integer)
_DF_ imFileReadImageInfo(ifile as imfile ptr, index as integer, w as integer ptr, h as integer ptr, file_color_mode as integer ptr, file_data_type as integer ptr) as integer
_DF_ imFileWriteImageInfo(ifile as imfile ptr, w as integer, h as integer, user_color_mode as integer, user_data_type as integer) as integer
_DF_ imFileReadImageData(ifile as imfile ptr, data as any ptr, convert2bitmap as integer, color_mode_flags as integer) as integer
_DF_ imFileWriteImageData(ifile as imfile ptr, data as any ptr) as integer
_DS_ imFormatRegisterInternal()
_DS_ imFormatRemoveAll()
_DS_ imFormatList(format_list as zstring ptr ptr, format_count as integer ptr)
_DF_ imFormatInfo(format as const zstring ptr, desc as zstring ptr, ext as zstring ptr, can_sequence as integer ptr) as integer
_DF_ imFormatCompressions(format as const zstring ptr, comp zstring ptr ptr, comp_count as integer ptr, color_mode as integer, data_type as integer) as integer
_DF_ imFormatCanWriteImage(format as const zstring ptr, compression as const zstring ptr, color_mode as integer, data_type as integer) as integer

end extern

#include once "old_im.bi"

#endif
