#ifndef __IUP_STR_BI__ 
#define __IUP_STR_BI__

#include once "iup.bi"


extern "c"

' Returns a non zero value if the two strings are equal.
' str1 or str2 can be NULL.
_DF_ iupStrEqual(str1 as const zstring ptr, str2 as const zstring ptr) as integer

' Returns a non zero value if the two strings are equal but ignores case.
' str1 or str2 can be NULL.
_DF_ iupStrEqualNoCase(str1 as const zstring ptr, str2 as const zstring ptr) as integer

' Returns a non zero value if the two strings are equal 
' up to a number of characters defined by the strlen of the second string.
' str1 or str2 can be NULL.
_DF_ iupStrEqualPartial(str1 as const zstring ptr, str2 as const zstring ptr) as integer

' Returns 1 if the string is "1", "YES", "ON" or "TRUE".
' Returns 0 otherwise.
_DF_ iupStrBoolean(str_ as const zstring ptr) as integer

' Returns 1 if the string is "NO", "OFF" or "FALSE".
' Returns 0 otherwise.
_DF_ iupStrFalse(str_ as const zstring ptr) as integer

' Returns the number of lines in a string.
' It works for UNIX, DOS and MAC line ends.
_DF_ iupStrLineCount(str_ as const zstring ptr) as integer

' Returns the a pointer to the next line and the size of the current line.
' It works for UNIX, DOS and MAC line ends. The size does not includes the line end.
' If str is NULL it will return NULL.
_DF_ iupStrNextLine(str_ as const zstring ptr, len_ as integer ptr) as const zstring ptr

' Returns the number of repetitions of the character occours in the string.
_DF_ iupStrCountChar(str_ as const zstring ptr, c as integer) as integer

' Returns a copy of the given string.
' If str is NULL it will return NULL.
_DF_ iupStrDup(str_ as const zstring ptr) as zstring ptr

' Returns a new string containing a copy of the string up to the character.
' The string is then incremented to after the position of the character.
_DF_ iupStrCopyUntil(ppStr as zstring ptr ptr, c as integer) as zstring ptr

' Copy the string to the buffer, but limited to the max_size of the buffer.
' buffer is always porperly ended.
_DS_ iupStrCopyN(dst_str as zstring ptr, dst_max_size as integer, src_str as const zstring ptr)

' Returns a buffer with the specified size+1. \n
' The buffer is resused after 50 calls. It must NOT be freed.
' Use size=-1 to free all the internal buffers.
_DF_ iupStrGetMemory(size as integer) as zstring ptr

' Returns a buffer that contains a copy of the given buffer using \ref iupStrGetMemory.
_DF_ iupStrGetMemoryCopy(str_ as const zstring ptr) as zstring ptr

' Converts a string into lower case. Can be used in-place.
_DS_ iupStrLower(dstr as zstring ptr, sstr as const zstring ptr)

' Converts a string into upper case. Can be used in-place.
_DS_ iupStrUpper(dstr as zstring ptr, sstr as const zstring ptr)

' Checks if the string has at least 1 space character.
_DF_ iupStrHasSpace(str_ as const zstring ptr) as integer

' Extract a RGB triple from the string. Returns 0 or 1.
_DF_ iupStrToRGB(str_ as const zstring ptr, r as ubyte ptr, g as ubyte ptr, b as ubyte ptr) as integer

' Extract a RGBA quad from the string, alpha is optional. Returns 0, 3 or 4.
_DF_ iupStrToRGBA(str_ as const zstring ptr, r as ubyte ptr, g as ubyte ptr, b as ubyte ptr, a as ubyte ptr) as integer

' Converts the string to an int. The string must contains only the integer value.
' Returns a a non zero value if sucessfull.
_DF_ iupStrToInt(str_ as const zstring ptr, i as integer ptr) as integer

' Converts the string to two int. The string must contains two integer values in sequence, 
' separated by the given character (usually 'x' or ':').
' Returns the number of converted values.
' Values not extracted are not changed.
_DF_ iupStrToIntInt(str_ as const zstring ptr, i1 as integer ptr, i2 as integer ptr, sep_one_asci as ubyte) as integer

' Converts the string to an float. The string must contains only the real value.
' Returns a a non zero value if sucessfull.
_DF_ iupStrToFloat(str_ as const zstring ptr, f as single ptr) as integer

' Converts the string to two float. The string must contains two real values in sequence, 
' separated by the given character (usually 'x' or ':').
' Returns the number of converted values.
' Values not extracted are not changed.
_DF_ iupStrToFloatFloat(str_ as const zstring ptr, f1 as single ptr, f2 as single ptr, sep_one_asci as ubyte) as integer

' Extract two strings from the string.
' separated by the given character (usually 'x' or ':').
' Returns the number of converted values.
' Values not extracted are not changed.
_DF_ iupStrToStrStr(str_ as const zstring ptr, str1 as zstring ptr, str2 as zstring ptr, sep_one_asci as ubyte) as integer

' Returns the file extension of a file name.
' Supports UNIX and Windows directory separators.
_DF_ iupStrFileGetExt(file_name as const zstring ptr) as zstring ptr

' Returns the file title of a file name.
' Supports UNIX and Windows directory separators.
_Df_ iupStrFileGetTitle(file_name as const zstring ptr) as zstring ptr

' Returns the file path of a file name.
' Supports UNIX and Windows directory separators.
_DF_ iupStrFileGetPath(file_name as const zstring ptr) as zstring ptr

' Concat path and title addind '/' between if path does not have it.
_DF_ iupStrFileMakeFileName(path as const zstring ptr, title as const zstring ptr) as zstring ptr

' Split the filename in path and title using pre-alocated strings.
_DS_ iupStrFileNameSplit(filename as const zstring ptr, path as zstring ptr, title as zstring ptr)

' Replace a character in a string.
' Returns the number of occurrences.
_DF_ iupStrReplace(str_ as zstring ptr, srcChar as ubyte, dstChar as ubyte) as integer

' Convert line ends to UNIX format in-place (one \n per line).
_DS_ iupStrToUnix(str_ as zstring ptr)

' Convert line ends to MAC format in-place (one \r per line).
_DS_ iupStrToMac(str_ as zstring ptr)

' Convert line ends to DOS/Windows format (the sequence \r\n per line).
' If returned pointer different the input, it must be freed.
_DF_ iupStrToDos(str_ as const zstring ptr) as zstring ptr

' Convert string to C format. Process \n, \r and \t.
' If returned pointer different the input, it must be freed.
_DF_ iupStrConvertToC(str_ as const zstring ptr) as zstring ptr

' Remove the interval from the string. Done in-place.
_DS_ iupStrRemove(value as zstring ptr, start as integer, end_ as integer, dir_ as integer)

' Remove the interval from the string and insert the new string at the start.
_DF_ iupStrInsert(value as const zstring ptr, insert_value as const zstring ptr, start as integer, end_ as integer) as zstring ptr

' Process the mnemonic in the string. If not found returns str.
' If found returns a new string. Action can be:
' 1: replace & by c
'-1: remove & and return in c
' 0: remove
_DF_ iupStrProcessMnemonic(str_ as const zstring ptr, c as zstring ptr, action as integer) as zstring ptr

end extern


#endif
