#ifndef __FBS_PLUGOUT_DS__
#define __FBS_PLUGOUT_DS__

'  ##############
' # plug-ds.bi #
'##############
' Copyright 2005 - 2014 by D.J.Peters (Joshy)
' d.j.peters@web.de

' test of secondary DirectSoundBuffer
#include once "windows.bi"       ' <- BOOL, NULL ...
#include once "win/mmsystem.bi"  ' <- WAVEFORMETEX
#include once "win/dsound.bi"    ' <- IDirectSound
#include once "crt/string.bi"    ' <- memcpy

#inclib "dxguid"
#inclib "ole32"

' GUID helper
function Guid2String(byval pGuid as LPGUID) as string
  if pGuid=0 then return "00000000-0000-0000-0000000000000000"
  dim as string tmp = hex(pGuid->Data1,8) & "-" & _
                      hex(pGuid->Data2,4) & "-" & _
                      hex(pGuid->Data3,4) & "-"
  for i as integer=0 to 7
     tmp &= hex(pGuid->Data4(i),2)
  next
  return tmp
end function

' WAVEFORMATEX helper
sub setWaveFormatEx(byref wf        as WAVEFORMATEX, _
                    byval nRate     as integer     , _
                    byval nBits     as integer     , _
                    byval nChannels as integer)
  if nChannels<2 then nChannels=1 else nChannels=2
  wf.wFormatTag     = WAVE_FORMAT_PCM
  wf.nChannels      = nChannels
  wf.nSamplesPerSec = nRate
  wf.wBitsPerSample = nBits
  wf.nBlockAlign    = (nBits\8)*nChannels
  wf.nAvgBytesPerSec=wf.nSamplesPerSec*wf.nBlockAlign
  wf.cbSize         =0
end sub

type DS_DEVICE_DESCRIPTOR
  public:
  declare destructor
  declare constructor(byval pGuid        as LPGUID, _
                      byval pDescription as LPCSTR, _
                      byval pModule      as LPCSTR)
  declare operator cast as string
  declare function getGuid as LPGUID
  declare function getDescription as zstring ptr
  private:
  as LPGUID m_pGuid
  as LPCSTR m_pDescription
  as LPCSTR m_pModule
end type
type PDS_DEVICE_DESCRIPTOR as DS_DEVICE_DESCRIPTOR ptr

destructor DS_DEVICE_DESCRIPTOR
  dprint("DS_DEVICE_DESCRIPTOR~")
  if m_pGuid then 
    delete m_pGuid
    m_pGuid=0
  end if
  if m_pDescription then
    delete[] m_pDescription
    m_pDescription=0
  end if
  if m_pModule then
    delete[] m_pModule
    m_pModule=0
  end if
end destructor

constructor DS_DEVICE_DESCRIPTOR(byval pGuid        as LPGUID, _ 
                                 byval pDescription as LPCSTR, _
                                 byval pModule      as LPCSTR)
  dprint("DS_DEVICE_DESCRIPTOR()")
  if (pGuid<>0) then
    m_pGuid=new GUID
    *m_pGUID=*pGuid
  end if
  if (pDescription<>NULL) then
    dim as integer nChars=len(*pDescription)+1
    m_pDescription=new ubyte[nChars]
    *m_pDescription=*pDescription
  end if
  if (pModule<>NULL) then
    dim as integer nChars=len(*pModule)+1
    m_pModule=new ubyte[nChars]
    *m_pModule=*pModule
  end if
end constructor

operator DS_DEVICE_DESCRIPTOR.cast as string
  dim as string tmp = Guid2String(m_pGuid) & " " & *m_pDescription
  operator = tmp
end operator

function DS_DEVICE_DESCRIPTOR.getGuid as LPGUID
  return m_pGuid
end function

function DS_DEVICE_DESCRIPTOR.getDescription as zstring ptr
  return m_pDescription
end function


type DS_DEVICE_LIST
  declare destructor
  as integer                   nDevices
  as PDS_DEVICE_DESCRIPTOR ptr  pDevices
end type
type PDS_DEVICE_LIST as DS_DEVICE_LIST ptr

destructor DS_DEVICE_LIST
  if (pDevices<>NULL) then
    if nDevices>0 then
      for i as integer=0 to nDevices-1
        if (pDevices[i]<>NULL) then
          delete pDevices[i]
          pDevices[i]=0
        end if
      next
      nDevices=0
    end if
    deallocate pDevices
    pDevices=0
  end if
  dprint("DS_DEVICE_LIST~")
end destructor


function DSDeviceEnumCB(byval pGuid as LPGUID, byval pDescription as LPCSTR, byval pModule as LPCSTR, byval pUserdata as LPVOID) as BOOL
  dim as DS_DEVICE_LIST ptr pList=cptr(DS_DEVICE_LIST ptr,pUserdata)
  if (pList=NULL) then 
    dprint("DSDeviceEnumCB error: pList=NULL !")
    return 0 ' interrupt enumeration
  end if
  ' the first primary (default) device does not have a GUID 
  ' (ignore it here it will be two times in the list)
  if pGUID=0 then return 1 ' get next device
  ' be sure it's not an emulated DirectSound device
  ' it's better to use the MM AudioDevice directly as an emulated DirectSound device
  dim as LPDIRECTSOUND ds
  ' craete the device interface if bussy try next device
  if (DirectSoundCreate(pGuid,@ds,NULL)<>DS_OK) then return 1
  dim as integer AddDeviceToList=0
  ' get device caps
  dim as DSCAPS caps
  caps.dwSize=sizeof(DSCAPS)
  if IDirectSound_GetCaps(ds,@caps)=DS_OK then
    ' not an emulated MM device ?
    if (caps.dwFlags and DSCAPS_EMULDRIVER)=0 then 
      ' 16 bit primary sound buffer ?
      if (caps.dwFlags and DSCAPS_PRIMARY16BIT) then
        ' stereo primary sound buffer ?
        if (caps.dwFlags and DSCAPS_PRIMARYSTEREO) then
          if (IDirectSound_SetCooperativeLevel(ds,GetForegroundWindow(),DSSCL_PRIORITY)=DS_OK) then
            AddDeviceToList=1
          end if
        end if
      end if
    end if
  end if
  ' free the interface
  IDirectSound_Release(ds)
  if AddDeviceToList<>0 then
    ' allocate new item in list
    pList->pDevices=reallocate(pList->pDevices,sizeof(PDS_DEVICE_DESCRIPTOR)*(pList->nDevices+1))
    pList->pDevices[pList->nDevices]=new DS_DEVICE_DESCRIPTOR(pGuid,pDescription,pModule)
    pList->nDevices+=1
  end if
  return 1 ' allow next device enumeration
end function

#endif ' __FBS_PLUGOUT_DS__